@@ -53,7 +53,7 @@ auto.msarima <- function(y, orders=list(ar=c(3,3),i=c(2,1),ma=c(3,3)), lags=c(1,
 
     return(adam(data, model="NNN", orders=orders, lags=lags, distribution="dnorm",
                 initial=initial, loss=loss, h=h, holdout=holdout, bounds=bounds,
-                silent=silent, regressors=regressors, ...))
+                silent=silent, regressors=regressors, adamETS=FALSE, ...))
 
 # Function estimates several msarima models and selects the best one using the selected information criterion.
 #
