@@ -14,7 +14,7 @@ List adamFitter(arma::mat &matrixVt, arma::mat const &matrixWt, arma::mat &matri
                 unsigned int const &nNonSeasonal, unsigned int const &nSeasonal,
                 unsigned int const &nArima, unsigned int const &nXreg, bool const &constant,
                 arma::vec const &vectorYt, arma::vec const &vectorOt, bool const &backcast,
-                unsigned int const &nIterations, bool const &refineHead){
+                unsigned int const &nIterations, bool const &refineHead, bool const &adamETS){
     /* # matrixVt should have a length of obs + lagsModelMax.
      * # matrixWt is a matrix with nrows = obs
      * # vecG should be a vector
@@ -70,8 +70,6 @@ List adamFitter(arma::mat &matrixVt, arma::mat const &matrixWt, arma::mat &matri
                 vecErrors(i-lagsModelMax) = 0;
             }
             else{
-                // We need this multiplication for cases, when occurrence is fractional
-                vecYfit(i-lagsModelMax) = vectorOt(i-lagsModelMax)*vecYfit(i-lagsModelMax);
                 vecErrors(i-lagsModelMax) = errorf(vectorYt(i-lagsModelMax), vecYfit(i-lagsModelMax), E);
             }
 
@@ -79,8 +77,14 @@ List adamFitter(arma::mat &matrixVt, arma::mat const &matrixWt, arma::mat &matri
             profilesRecent(indexLookupTable.col(i)) = adamFvalue(profilesRecent(indexLookupTable.col(i)),
                            matrixF, E, T, S, nETS, nNonSeasonal, nSeasonal, nArima, nComponents, constant) +
                 adamGvalue(profilesRecent(indexLookupTable.col(i)), matrixF, matrixWt.row(i-lagsModelMax), E, T, S,
-                           nETS, nNonSeasonal, nSeasonal, nArima, nXreg, nComponents, constant, vectorG, vecErrors(i-lagsModelMax));
+                           nETS, nNonSeasonal, nSeasonal, nArima, nXreg, nComponents, constant,
+                           vectorG, vecErrors(i-lagsModelMax), vecYfit(i-lagsModelMax), adamETS);
 
+            // If ot is fractional, amend the fitted value
+            if(vectorOt(i-lagsModelMax)!=0 && vectorOt(i-lagsModelMax)!=1){
+                // We need this multiplication for cases, when occurrence is fractional
+                vecYfit(i-lagsModelMax) = vectorOt(i-lagsModelMax)*vecYfit(i-lagsModelMax);
+            }
         }
 
         ////// Backwards run
@@ -115,7 +119,7 @@ List adamFitter(arma::mat &matrixVt, arma::mat const &matrixWt, arma::mat &matri
                                    adamGvalue(profilesRecent(indexLookupTable.col(i)), matrixFInv,
                                               matrixWt.row(i-lagsModelMax), E, T, S,
                                               nETS, nNonSeasonal, nSeasonal, nArima, nXreg, nComponents, constant,
-                                              vectorGInv, vecErrors(i-lagsModelMax));
+                                              vectorGInv, vecErrors(i-lagsModelMax), vecYfit(i-lagsModelMax), adamETS);
             }
 
             // Fill in the head of the series.
@@ -156,14 +160,14 @@ RcppExport SEXP adamFitterWrap(arma::mat matrixVt, arma::mat &matrixWt, arma::ma
                                unsigned int const &componentsNumberETS, unsigned int const &nSeasonal,
                                unsigned int const &nArima, unsigned int const &nXreg, bool const &constant,
                                arma::vec &vectorYt, arma::vec &vectorOt, bool const &backcast,
-                               unsigned int const &nIterations, bool const &refineHead){
+                               unsigned int const &nIterations, bool const &refineHead, bool const &adamETS){
 
     unsigned int nNonSeasonal = componentsNumberETS - nSeasonal;
 
     return wrap(adamFitter(matrixVt, matrixWt, matrixF, vectorG,
                            lags, indexLookupTable, profilesRecent, Etype, Ttype, Stype,
                            nNonSeasonal, nSeasonal, nArima, nXreg, constant,
-                           vectorYt, vectorOt, backcast, nIterations, refineHead));
+                           vectorYt, vectorOt, backcast, nIterations, refineHead, adamETS));
 }
 
 /* # Function produces the point forecasts for the specified model */
@@ -233,11 +237,14 @@ arma::mat adamErrorer(arma::mat const &matrixVt, arma::mat const &matrixWt, arma
     for(unsigned int i = 0; i < (obs-horizon); i=i+1){
         hh = std::min(horizon, obs-i);
         // Update the profile to get the recent value from the state matrix
-        profilesRecent(indexLookupTable.col(i+lagsModelMax-1)) = matrixVt.col(i+lagsModelMax-1);
+        // lagsModelMax moves the thing to the next obs. This way, we have the structure
+        // similar to the fitter
+        profilesRecent(indexLookupTable.col(i+lagsModelMax)) = matrixVt.col(i+lagsModelMax);
         // profilesRecent(indexLookupTable.col(i)) = matrixVt.col(i);
         // This also needs to take probability into account in order to deal with intermittent models
         matErrors.submat(0, i, hh-1, i) = (errorvf(vectorYt.rows(i, i+hh-1),
                                            adamForecaster(matrixWt.rows(i,i+hh-1), matrixF,
+                                                          // lags, indexLookupTable.cols(i,i+hh-1), profilesRecent,
                                                           lags, indexLookupTable.cols(i+lagsModelMax,i+lagsModelMax+hh-1), profilesRecent,
                                                           E, T, S, nNonSeasonal, nSeasonal, nArima, nXreg, constant, hh), E));
     }
