@@ -14,17 +14,17 @@ test_that("ADAM ETS(ZZZ) selection on AirPassengers", {
 })
 
 # Full ADAM selection
-test_that("ADAM ETS(FFF) selection on BJsales", {
+test_that("ADAM ETS(PPP) selection on BJsales", {
     skip_on_cran()
-    testModel <- adam(BJsales, "FFF")
+    testModel <- adam(BJsales, "PPP")
     expect_match(modelType(testModel), "AAdN")
 })
 
 # ADAM with specified pool
 test_that("ADAM selection with a pool on BJsales", {
     skip_on_cran()
-    testModel <- adam(BJsales, c("AAA","ANN","MAN","MAM"))
-    expect_match(modelType(testModel), "MAN")
+    testModel <- adam(BJsales, c("ANN","MNN","AAN","MAN"))
+    expect_match(modelType(testModel), "AAN")
 })
 
 # ADAM forecasts with simulated interval
