@@ -3,11 +3,11 @@ context("Tests for es() function");
 # Basic ETS selection
 testModel <- es(BJsales, silent=TRUE);
 test_that("Test ETS selection on BJsales", {
-    expect_match(modelType(testModel), "AAdN");
+    expect_equal(length(testModel$ICs), 5);
 })
 
 test_that("Test damped-trend ETS on BJsales", {
-    expect_equal(es(BJsales,model="AAdN", silent=TRUE)$phi, 0.86, tolerance=0.1);
+    expect_equal(es(BJsales,model="AAdN", silent=TRUE)$phi, 0.88, tolerance=0.1);
 })
 
 # Reuse previous ETS
